#ifndef HARKIO_HARKCONFIG_IMPULSERESPONSELILSTFILETEXTHELPER_HPP_
#define HARKIO_HARKCONFIG_IMPULSERESPONSELILSTFILETEXTHELPER_HPP_

#include <ostream>
#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>
#include <harkio2/AnalyzeError.hpp>
#include <harkio2/harkconfig/HarkConfigFileTextHelper.hpp>

/** @file ImpulseResponseListFileTextHelper.hpp
 * @brief ImpulseResponseListFileTextHelper class definition.
 */

namespace harkio2
{
namespace harkconfig
{

class HarkConfigFile;
class ImpulseResponseListFile1_0;
class Source1;

/** @brief HarkConfigFileTextReader のためのヘルパークラス。
 *
 * このクラスは HarkConfigFileTextReader が ImpulseResponseListFile を読み込むのを助ける。
 * 内部的に ImpulseResponseListFile1_0TextHelper を用いる。
 * 将来的に Impulse Response List File のフォーマットが刷新されたときは
 * ImpulseResponseListFile2_0TextHelper のようなクラスを作る必要があるだろう。
 */
class ImpulseResponseListFileTextHelper : public HarkConfigFileTextHelper
{
public:

    /** @brief Returns a constructed HarkConfigFile structure.
     * Returns a HarkConfigFile structure constructed by calling AnalyzeLine() till now.
     * If there is no AnalyzeLine() calling, returns empty pointer.
     * @return A constructed HarkConfigFile.
     */
    virtual boost::shared_ptr<HarkConfigFile> GetData();

    /** @brief Inputs a line and updates a inside HarkConfigFile structure.
     * Inputs a line which was read from a file, and updates a inside HarkConfigFile structure.
     * @param line A line which was read from a file
     * @param lineNum The line number of 'line' (0:unknown line number)
     */
    virtual void AnalyzeLine(const std::string& line, unsigned int lineNum = 0);

    /** @brief Writes a inside HarkConfigFile to a output stream.
     * @param out A output stream to write data.
     */
    virtual void Write(std::ostream& out);

private:
    /// @brief Helper class which absorbs version difference.
    boost::shared_ptr<HarkConfigFileTextHelper> versionHelper;
};

/** @brief Helper class for HarkConfigFileTextReader.
 * This class helps HarkConfigFileTextReader to read ImpulseResponseListFile1_0
 */
class ImpulseResponseListFile1_0TextHelper : public HarkConfigFileTextHelper
{
public:
    /** @brief Constructs a helper
     */
    ImpulseResponseListFile1_0TextHelper();

    /** @brief Returns a constructed HarkConfigFile structure.
     * Returns a HarkConfigFile structure constructed by calling AnalyzeLine() till now.
     * If there is no AnalyzeLine() calling, returns empty pointer.
     * @return A constructed HarkConfigFile.
     */
    virtual boost::shared_ptr<HarkConfigFile> GetData();

    /** @brief Inputs a line and updates a inside HarkConfigFile structure.
     * Inputs a line which was read from a file, and updates a inside HarkConfigFile structure.
     * @param line A line which was read from a file
     * @param lineNum The line number of 'line' (0:unknown line number)
     */
    virtual void AnalyzeLine(const std::string& line, unsigned int lineNum = 0);

    /** @brief Writes a inside HarkConfigFile to a output stream.
     * @param out A output stream to write data.
     */
    virtual void Write(std::ostream& out);

private:
    /// @brief The data structure which is being constructed by AnalyzeLine().
    boost::shared_ptr<ImpulseResponseListFile1_0> data;
    /// @brief The source array of ImpulseResponseListFile.
    boost::shared_ptr<std::vector<Source1> > sources;
};

}
}

#endif // HARKIO_HARKCONFIG_IMPULSERESPONSELILSTFILETEXTHELPER_HPP_

