#include <harkio2/hark/HarkFileBinaryWriter.hpp>

#include <boost/static_assert.hpp>
#include <boost/format.hpp>
#include <harkio2/hark/HarkFile.hpp>

// load Helper implementation.
#include "HarkFileBinaryWriter/Helper.cpp"

using namespace std;
using namespace boost;

BOOST_STATIC_ASSERT(sizeof(float) == 4);

namespace harkio2
{
namespace hark
{

HarkFileBinaryWriter::HarkFileBinaryWriter(std::ostream& out)
    : helper(new Helper(*this)), out(&out), temp_oss()
{ }

HarkFileBinaryWriter::~HarkFileBinaryWriter()
{ }

void HarkFileBinaryWriter::Write(boost::shared_ptr<harkio2::hark::HarkFile> hark) throw (harkio2::InvalidHarkFile)
{
    // check hark file
    if (hark)
    {
        Write(*hark);
    }
}

void HarkFileBinaryWriter::Write(const harkio2::hark::HarkFile& o) throw (harkio2::InvalidHarkFile)
{
    const char* funcName = "HarkFileBinaryWriter::Write";
    typedef boost::shared_ptr<harkio2::hark::DataSet> DataSetPtr;

    if (o.GetDataSetsSize() == 0)
    {
        return;
    }
    else if (!IsValidHarkFile(o))
    {
        throw harkio2::InvalidHarkFile((format("%s: invalid hark file") % funcName).str());
    }

    if (o.GetDataSet("hgtf::Fmt"))
    {
        std::ostream* temp = this->out;
        this->out = &this->temp_oss;

        helper->Write(o);

        this->out = temp;

        this->out->write("HARK", 4);
        helper->Write<uint32_t>(temp_oss.str().length(), *this->out);
        (*this->out) << temp_oss.str();

        temp_oss.str("");

    }
    else
    {
        throw harkio2::InvalidHarkFile((format("%s: hark file must contain a 'fmt' dataset") % funcName).str());
    }

}

void HarkFileBinaryWriter::Write(const harkio2::hark::hgtf::Fmt1_0& o) throw (harkio2::InvalidHarkFile) { helper->Write(o); }
void HarkFileBinaryWriter::Write(const harkio2::hark::hgtf::Spec1_0& o) throw (harkio2::InvalidHarkFile) { helper->Write(o); }
void HarkFileBinaryWriter::Write(const harkio2::hark::hgtf::M2PG1_0& o) throw (harkio2::InvalidHarkFile) { helper->Write(o); }
void HarkFileBinaryWriter::Write(const harkio2::hark::hgtf::GTF1_0& o) throw (harkio2::InvalidHarkFile) { helper->Write(o); }
void HarkFileBinaryWriter::Write(const harkio2::hark::hgtf::SM1_0& o) throw (harkio2::InvalidHarkFile) { helper->Write(o); }

}
}

