#define BOOST_TEST_MODULE "testcase for Endian class"
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#include <stdint.h>
#include <harkio2/Endian.hpp>

using harkio2::Endian;

BOOST_AUTO_TEST_CASE(endian_checker_test)
{
    uint32_t hostValue = 0x01020304;
    bool hostIsLittle = reinterpret_cast<char*>(&hostValue)[0] == 0x04;

    BOOST_CHECK_EQUAL(hostIsLittle, Endian::HostIsLittle());
}

BOOST_AUTO_TEST_CASE(operators_test)
{
    char littleOf1234_[4] = { 4, 3, 2, 1 };
    char bigOf1234_[4] = { 1, 2, 3, 4 };
    uint32_t littleOf1234, bigOf1234;

    memcpy(reinterpret_cast<void*>(&littleOf1234), littleOf1234_, 4);
    memcpy(reinterpret_cast<void*>(&bigOf1234), bigOf1234_, 4);

#if defined(BOOST_LITTLE_ENDIAN)
    BOOST_CHECK_EQUAL(0x01020304, littleOf1234);
    BOOST_CHECK_EQUAL(0x04030201, bigOf1234);
#elif defined(BOOST_LITTLE_ENDIAN)
    BOOST_CHECK_EQUAL(0x04030201, littleOf1234);
    BOOST_CHECK_EQUAL(0x01020304, bigOf1234);
#else
#error PDP endian is not supported
#endif

    BOOST_CHECK_EQUAL(0x01020304, Endian::LittleToHost(littleOf1234));
    BOOST_CHECK_EQUAL(Endian::HostToLittle(0x01020304), littleOf1234);

    BOOST_CHECK_EQUAL(0x01020304, Endian::BigToHost(bigOf1234));
    BOOST_CHECK_EQUAL(Endian::HostToBig(0x01020304), bigOf1234);
}

